################################################################################
#    R LABORATORY                                                              #
#                                                                              #
#    Lesson 5 October 2018                                                     #
#                                                                              #
#    If you have any questions please contact me at:                           #
#    francesco.schirripa@ec.unipi.it                                           #
#                                                                              #
#    Room 206 2^nd floor, DEM                                                #
#                                                                              #
################################################################################

#-------------------------------------------------------------------------------
# R as a calculator

3+5

3*5

5/5

6-2

sqrt(36)

#-------------------------------------------------------------------------------
# All text after the pound sign "#" within the same line is considered a comment.
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#The online help is a very useful tool to familiarise with R and its commands:

help.start()          #opens the html main page of R online help;
help(sqrt)            #details about the function 
?sqrt
help.search("linear models")
??linearmodels        #performs an online search based on keyword;
apropos("mean")       #returns all the commands containing "mean" in its name.

#-------------------------------------------------------------------------------
#Object class
x <- 1
y <- 2.5
class(x)
class(y)
class(x+y)

#-------------------------------------------------------------------------------
#Basic functions
getwd()          # current working directory
dir()            # displays all the files in the directory;

# REMEMBER change your working directory: choose the directory where you 
# have saved the data to be loaded)

################################################################################
#
#                               Vectors                                        #
#
################################################################################

x<-c(2, 3, 5)   #vector containing three numeric values 2, 3 and 5
x
str(x)          #display the internal structure of an R object

# !!!IMPORTANT!!!!
#Note that brackets [ ] are used for indexing, 
#whereas parentheses ( ) are used to call a function

x[2]            #select a specific element

# A vector can contain character strings.
#Character strings are entered using either matching double (" ") or single (' ') quotes, 
#but are printed using double quotes
b=c("aa", "bb")
b1=c('aa', 'bb')
b1
b
class(b)
str(b)

xb<-c(x,b)      #concatenation of two vectors
class(xb)
str(xb)
length(x)       #length of the vector


#A vector of 10 elements from 1 to 10
x<-seq(1,10,1)
#the same output can be obtain using
y<-c(1:10)

odd<-seq(1,10,2)  #step size equal to 2: obtaing the odd numbers between 1 and 10

x<-rep(8,5)   #repeat 8 five times

#I can repeat ax times a vector
x<-c(4,8)
xr<-rep(x,2)

#You can access an individual element of a vector by its position (or "index"),
#indicated using square brackets

color=c("red","orange","yellow","green","blue","indigo","violet")
color[2]        #selects the second element of the vector color

xr[1:3]


#-------------------------------------------------------------------------------
#      mathematical operations on vectors
#-------------------------------------------------------------------------------

a = c(1, 3, 5, 7)
b = c(1, 2, 4, 8)

#if we multiply a by 5, we would get a vector with each of its members multiplied by 5.
5 * a
#[1]  5 15 25 35

#if we add a and b together, the sum would be a vector whose members are the sum
# of the corresponding members from a and b.
a + b
#[1]  2  5  9 15

#Similarly for subtraction, multiplication and division, we get new vectors via
#memberwise operations.
a - b
#[1]  0  1  1 -1

a * b
#[1]  1  6 20 56

a / b
#[1] 1.000 1.500 1.250 0.875


a^2 # square of elements
#[1] 1  9 25 49

#If two vectors are of unequal length, the shorter one will be recycled in order
#to match the longer vector. For example, the following vectors u and v have
#different lengths, and their sum is computed by recycling values of the shorter vector u.
u = c(10, 20, 30)
v = c(1, 2, 3, 4, 5, 6, 7, 8, 9)
u + v
#[1] 11 22 33 14 25 36 17 28 39

#summing the elements of a vector
sum(v)
#[1] 45

#-------------------------------------------------------------------------------
#               statistical operations
#-------------------------------------------------------------------------------

s1 <- c (6, 1, 5, 9, 4, 7, 8, 2, 5, 8)
mean(s1) #mean

median(s1) # median

range(s1) # range (min and max)

sd(s1)    #standard deviation

summary(s1) #provide summary stats about s1
#summary() is a very useful multipurpose function in R (where X can be one of
#any number of objects). The summary function has different outputs depending on
#what kind of object it takes as an argument. We'll learn more in the next lessons



#-------------------------------------------------------------------------------
#               Logical values & operators
#-------------------------------------------------------------------------------

a<-c(6,2,5,3,8,2)
b<-c(1,4,3,6,7,2)

a<b
#[1] FALSE  TRUE FALSE  TRUE FALSE FALSE

a<=b
#[1] FALSE  TRUE FALSE  TRUE FALSE  TRUE

a!=b
#[1]  TRUE  TRUE  TRUE  TRUE  TRUE FALSE

is.element(2,a)
#[1] TRUE

is.element(9,a)
#[1] FALSE


#------>  MISSING VALUES

#In R, missing values are represented by the symbol NA (not available)

is.na(a)   # returns TRUE if an element of a is missing
#[1] FALSE FALSE FALSE FALSE FALSE FALSE

a[3]=NA      #assign a missing value at the second element of a
is.na(a)
#[1] FALSE  TRUE FALSE FALSE FALSE FALSE


#Re-coding particular values as NA (or any other value)

# Re-coding the occurrence of a particular value
 x = c(3.14, 98, 0, 99, 7, NA, 0, 99)
 x[x==99] = NA # Re-code all 99 in x as NA

# Re-coding the occurrence of NA
 x[is.na(x)] = -1 # Recode all NA in x as -1.
####-------------------------------------------

a[(a>7) | (a<5)]
#[1] 2 3 8 2

which (a<5)
#[1]  4 6



#-------------------------------------------------------------------------------
#                  Factor & lists
#-------------------------------------------------------------------------------

#define a numerical vector status
status<-c(rep(0,5), rep(1,8), rep(2,3))
status_factor<-factor(status, labels = c("divorced", "married", "single"))

levels(status_factor)
# [1] "divorced" "married"  "single"

table(status)       #table() gives the frequency table
#status
#0 1 2
#5 8 3

table(status_factor)
#status_factor
#divorced  married   single
#       5        8        3

#relevel()change the order of the level of a facotr and
#make a particular level first in the list
relev_status<-relevel(status_factor, ref="single")
table(relev_status)
#relev_status
#single divorced  married
#     3        5        8

n = c(2, 3, 5)                           
s = c("aa", "bb", "cc", "dd", "ee")
b = c(TRUE, FALSE, TRUE, FALSE, FALSE)

class(n)
class(s)
class(b)

mylist = list(n, s, b, 3)   # x contains copies of n, s, b
mylist



#-------------------------------------------------------------------------------
#                  Matrices & arrays
#-------------------------------------------------------------------------------
A = matrix(
c(1, 2, 3, 8, 8, 7), # the data elements
nrow=2,              # number of rows
ncol=3)              # number of columns
# by default matrix is filled by columns.
# I can fill the matrix by row using the argument:byrow = TRUE in the function matrix()
# A_bis = matrix( c(1, 2, 3, 8, 8, 7), nrow=2,ncol=3, byrow=T)   


A                       # print the matrix
#     [,1] [,2] [,3]
#[1,]    2    4    3
#[2,]    1    5    7


#INDEIXING: remember
# square brackets [ ] are used for indexing, 
# roud brackets ( ) are used to call a function

A[2, 3]      # element at 2nd row, 3rd column
A[2, ]       # the 2nd row
A[ ,3]       # the 3rd column
A[ ,c(1,3)]  # the 1st and 3rd columns


#assign names at columns and rows
dimnames(A) = list(
   c("row1", "row2"),         # row names
   c("col1", "col2", "col3")) # column names

A
A["row2", "col3"] # element at 2nd row, 3rd column

# ARRAY
# Create two vectors of different lengths.
vector1 <- c(5,9,3)
vector2 <- c(10,11,12,13,14,15)

# Take these vectors as input to the array.
result <- array(c(vector1,vector2),dim = c(3,3,2))
result


#linear algebra operations on a matrix
A = matrix( c(1, 2, 3, 8, 8, 7), nrow=2,ncol=3)      

B = matrix(
   c(2, 4, 3, 1, 5, 7),
   nrow=3,
   ncol=2)


A%*%B

C = matrix(
    c(2, 4, 3, 1),
    nrow=2,
    ncol=2)
det(C)       #determinant
t(C)         #matrix/vector transposition
solve(C)     #inverse
diag(C)      #extracts the diagonal of a matrix or builds a diagonal matrix.
diag(10,3,3) #build a diagonal matrix 3x3



#-------------------------------------------------------------------------------
#                            Dataframes
#-------------------------------------------------------------------------------

height = c(200, 165, 150)                      # class: numeric
patent= factor(c(1,1,2),labels=c("no", "yes")) # class: factor
gender = c("male", "female", "female")         # class: character

df = data.frame(height, patent, gender)       # df is a data frame

#some operations on dataframe using the built-in data frames in R 'iris'
help("iris")
str(iris)    #description of the dataframe
head(iris)   #returns the first 6 rows of a dataframe; head(iris,10) first 10rows
names(iris)  #variables' names
mean(iris$Petal.Length)  #mean of a single variable(column)
summary(iris)            #summary statistics on the entire dataframe

quantile(iris$Petal.Lengt,probs=0.99)
var(iris$Petal.Lengt)
cor(iris$Petal.Lengt, iris$Petal.Width)
plot(iris$Petal.Lengt, iris$Petal.Width,
     xlab = "Petal Lengt", ylab = "Petal Width", main = "Scatter plot")

table(iris$Species)                  #frequency distribution
prop.table(table(iris$Species))      #relative frequency distributions

attach(iris)
detach(iris)

# ordering dataframe by a given variable
iris_new<-iris[order(iris$Petal.Length),]

#add variable to dataframe
member=c(rep("Francesco",70),rep("Chiara", 80))
iris$member<-member

#delete variable
iris$member<- NULL

#tapply(X,INDEX,FUN) computes FUN for subsets of X (a vector) defined by INDEX.
#In this case I want the mean of Petal Lengt by Species
tapply(iris$Petal.Length,iris$Species,mean)

#the same results can be obtained with the function aggregate()
aggregate(iris$Petal.Length,list(iris$Species),mean)


#selecting a subset of observations
new_iris=subset(iris, iris$Petal.Length>quantile(iris$Petal.Length, probs=0.25))


#ADD ROWS (OBSERVATIONS) TO A DATAFRAME
df1<-data.frame(x=2:10, y=12:20)
df2<-data.frame(x=60:100,y=160:200)
df3<-rbind(df1, df2)

#MERGE 2 DATAFRAMES
df1<-data.frame(ID.1=1:10, y=21:30)
df2<-data.frame(ID.2=1:10, z=61:70)

df.merge<-merge(df1, df2, by.x="ID.1", by.y="ID.2")
# by.x and by.y specifications of the columns used for merging


#-------------------------------------------------------------------------------
#                   Importing data  and Exporting dataSe
#-------------------------------------------------------------------------------
#set the directory where my files are located

#SET WD!!!!!

#setwd("your WD")

#you can cahnge wd from the menu->Session->Set Working Directory->Choose Directory

mydata <- read.table("occupation.txt", header=TRUE, sep="" , dec=".")

#if I do not specify the working directory I have to specify the path of the data
#mydata2 <- read.table("C:\\Users\\franc\\OneDrive\\Documenti\\WEB PISA\\R Lab\\Data\\occupation.txt",
#                      header=TRUE, sep="" , dec=".")

mycsv<-read.csv("example 1.csv", sep=";" ,dec=",",header=TRUE)

View(mycsv)
str(mycsv)
dim(mycsv)
#detect the presence of NA
any(is.na(mycsv))
#delete cases with NA
mycsv2<-na.omit(mycsv)
dim(mycsv2)

#when you use mean() and you have missing value you have to set the option na.rm=T
mean(mycsv$y, na.rm=T)

write.table(mycsv2, "export1.txt", row.names = F)
#row.names = F: we do not want a character vector of row names

write.csv(mycsv2, "export1.csv", row.names = F)



